from django.db import models
from uuid import uuid4
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext as _
from django.views.generic import DetailView, DeleteView, UpdateView, ListView

operation_choices = (
    ('detail', _('Detail view, directly targeted')),
    ('list', _('List view, was part of the result list for a list/search view')),
    ('update', _('Update view, the object was displayed as part of editing the object.')),
    ('delete', _('Delete view, the object was displayed as part of deleting the object.'))
)


class ReadLog(models.Model):
    uuid = models.UUIDField(editable=False, blank=True, primary_key=True, default=uuid4)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_uuid = models.UUIDField()
    content_object = GenericForeignKey('content_type', 'object_uuid')
    user = models.ForeignKey(get_user_model(), on_delete=models.PROTECT)
    operation = models.TextField(choices=operation_choices)
    logged_at = models.DateTimeField(auto_now_add=True)

    @classmethod
    def generate_read_log_entries(cls, view, content_objects, user):
        """
        :param view: instance of the view
        :param content_objects: queryset or list of objects that needs to be logged
        :param user: user or anon user that needs to be logged
        """
        operation = getattr(view, 'log_operation', None)
        # allows overriding the log_operation directly on the view.
        if operation is None:
            if isinstance(view, DetailView):
                operation = 'detail'
            elif isinstance(view, UpdateView):
                operation = 'update'
            elif isinstance(view, DeleteView):
                operation = 'delete'
            elif isinstance(view, ListView):
                operation = 'list'
            else:
                operation = 'unknown'
        
        for content_object in content_objects:
            cls.objects.create(content_object=content_object, operation=operation, user=user)



# Generated by Django 2.2.7 on 2020-04-02 13:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ReadLog',
            fields=[
                ('uuid', models.UUIDField(blank=True, default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('object_uuid', models.UUIDField()),
                ('operation', models.TextField(choices=[('detail', 'Detail view, directly targeted'), ('list', 'List view, was part of the result list for a list/search view'), ('update', 'Update view, the object was displayed as part of editing the object.'), ('delete', 'Update view, the object was displayed as part of deleting the object.')])),
                ('logged_at', models.DateTimeField(auto_now_add=True)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



import os
import re
from setuptools import find_packages, setup


def read(f):
    return open(f, 'r', encoding='utf-8').read()


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('read_log')


setup(
    name='django-read-log',
    version=version,
    url='https://www.django-rest-framework.org/',
    license='MIT',
    description='Read log audit for django ',
    long_description=read('README.rst'),
    long_description_content_type='text/x-rst',
    author='Mikkel Aleister Clausen',
    author_email='mac@magenta.dk',
    packages=find_packages(exclude=['test_read_log']),
    include_package_data=True,
    install_requires=["django>=2.2"],
    python_requires=">=3.5",
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet :: WWW/HTTP',
    ],
    project_urls={
        'Source': 'https://github.com/magenta-aps/django-read-log',
        'Company': 'https://www.magenta.dk/'
    },
)


from read_log.models import ReadLog
from itertools import chain


class ReadLogMixin(object):
    log_operation = None

    def get_context_data(self, *args, object_list=None, **kwargs):
        context = super(ReadLogMixin, self).get_context_data(*args, object_list=object_list, **kwargs)
        objects = context.get('object_list', [])
        instance = context.get('object', None)
        if instance:
            if objects:
                objects = chain(objects, instance)
            else:
                objects = [instance]

        if objects:
            ReadLog.generate_read_log_entries(self, objects, self.request.user)

        return context
